<?php
/**
 * User: vetal
 * Date: 28.12.15
 * Time: 10:49
 */

include_once SIMPLYBOOK_DIR."/base/class/json-rpc-client.php";
include_once SIMPLYBOOK_DIR."/base/class/api-client.php";



try {
	//$protocol = isset( $_SERVER['HTTPS'] ) ? 'https://' : 'http://';
	$protocol = 'https://';

	$acc_protocols = stream_get_wrappers();

	if(!in_array('https', $acc_protocols) && in_array('http', $acc_protocols)){
		$protocol = 'http://';
	} elseif(!in_array('https', $acc_protocols) && !in_array('http', $acc_protocols)){
		throw new Exception('{"error":{"code":-99999,"message":"http and https protocols not supported","data":null},"id":"1"}');
	}

	$client = new ApiClient($protocol . $simplybook_cfg['url'] . '/', $simplybook_cfg['login'], $simplybook_cfg['api_key'] );

	if ( false === ( $available_methods = get_transient( 'simplybook_available_methods' ) ) ) {
		$available_methods = json_decode( file_get_contents( $protocol . $simplybook_cfg['url'] ) );
		$available_methods = array_keys( (array) $available_methods->services );
		set_transient( 'simplybook_available_methods', $available_methods, 24 * HOUR_IN_SECONDS );
	}

	$method = sanitize_text_field($_REQUEST['apimethod']); //sanitized
	$params = ( isset( $_REQUEST['params'] ) && ! empty( $_REQUEST['params'] ) ) ?  (array)$_REQUEST['params'] : array();

	$params = array_map_recursive( 'esc_attr', $params ); //sanitized

	if (in_array($method, $simplybook_cfg['cached_methods'])) {  //if cached method

		$key = md5(serialize($params));
		$cache_data = get_transient( 'simplybook_'.$method );

		if ( !isset($cache_data[$key]) ) {
			$cache_data[$key] =  $client->__call( $method, $params ) ;
			set_transient( 'simplybook_'.$method, $cache_data, 24 * HOUR_IN_SECONDS );
		}

		echo json_encode($cache_data[$key]);
	} else
	if($method == 'getAllList') {   // getAllList
		$buffmethods = array();
		$buffparams = array();

		$list_arr = array(
			"getLocationsList"  =>  array(true),
			"getCategoriesList" =>  array(),
			"getEventList"  =>  array(),
			"getUnitList"   =>  array(),
			"getCompanyParam"   =>  array('max_group_bookings')
		);

		$pl_list = array('any_unit','location',	'event_category', 'group_booking');

		$pl_status = $client->__call( 'getPluginStatuses', array($pl_list) );


		if(!$pl_status->location){
			unset($list_arr['getLocationsList']);
		}

		if(!$pl_status->event_category){
			unset($list_arr['getCategoriesList']);
		}

		if(!$pl_status->group_booking){
			unset($list_arr['getCompanyParam']);
		}

		$res = array();

		foreach($list_arr as $curr_method=>$curr_params){
			$key = md5(serialize($curr_params));
			$res[$curr_method] = get_transient( 'simplybook_'.$curr_method );

			if ( !isset($res[$curr_method][$key]) ) {
				array_push($buffmethods, $curr_method);
				array_push($buffparams, $curr_params);
				//$res[$curr_method] =  $client->__call( $curr_method, $curr_params ) ;
				//set_transient( 'simplybook_'.$curr_method, array($key => $res[$curr_method]), 24 * HOUR_IN_SECONDS );
			} else {
				$res[$curr_method] = $res[$curr_method][$key];
			}
		}

		if(count($buffmethods)>0){
			$newres = $client->__call( $buffmethods, $buffparams );

			foreach($newres as $curr_method=>$curr_res){
				$res[$curr_method] =  $curr_res;
				$key = md5(serialize($list_arr[$curr_method]));
				set_transient( 'simplybook_'.$curr_method, array($key => $res[$curr_method]), 24 * HOUR_IN_SECONDS );
			}
		}

		echo json_encode($res);
	} else
	if($method == 'getClientTimezoneOffset') {  //getClientTimezoneOffset

		$company_timezone = get_transient( 'simplybook_company_timezone');

		if ( !$company_timezone ) {
			$company_timezone = $client->__call('getCompanyParam',array('company_timezone'));
			set_transient( 'simplybook_company_timezone', $company_timezone, 24 * HOUR_IN_SECONDS );
		}

		$company_dtz = new DateTimeZone($company_timezone);
		$company_dt = new DateTime("now", $company_dtz);

		$client_offset = (int)$params[0]; //in minutes
		$company_offset = $company_dtz->getOffset($company_dt)/60; //in minutes

		echo json_encode(array(
			'company_timezone' => $company_timezone,
			'company_offset' => $company_offset,
			'client_offset' => $client_offset,
			'offset' => $company_offset - $client_offset
		));

	} else
	if($method == 'confirmDelayPayment') {  //confirmDelayPayment

		$allow_delay = $client->__call('getCompanyParam',array('allow_delay_payment'));

		if($allow_delay && is_array($params[0])){
			$bookings = array_map ('intval', $params[0]);
			$cart_res = (array)$client->__call('getBookingCart', array($bookings));
			$sign = md5($cart_res['cart_id'] .  $cart_res['cart_hash']. $simplybook_cfg['api_secret_key']);

			echo json_encode($client->__call('confirmBookingCart', array( $cart_res['cart_id'], 'delay', $sign) ));
		} else{
			echo json_encode(false);
		}
	} else
	if($method == 'cancelBooking') {  //confirmDelayPayment
		$booking_id = (int)$params[0];
		$in_book = false;

		$bookings = json_decode($_SESSION['book'])->bookings;

		foreach($bookings as $el){

			if($booking_id == (int)$el->id){
				$in_book = true;
				$sign = md5($booking_id . $el->hash . $simplybook_cfg['api_secret_key']);
				echo json_encode($client->__call('cancelBooking',array($booking_id, $sign)));
			}
		}

		if(!$in_book){
			echo json_encode(false);
		}

	} else
	if ( in_array( $method, $available_methods ) ) {  //non cached methods
		echo $result = json_encode( $client->__call( $method, $params ) );

		if($method == 'book'){
			$_SESSION['book'] = $result;
			$_SESSION['book_params'] = $params;
		}

		if($method == 'getPluginStatuses'){
			$last_result = get_transient('simplybook_plugins_status');

			if(isset($last_result) && $last_result != $result ){ //if plugin status changed - clear cache
				simplybook_clear_cache();
			}

			set_transient( 'simplybook_plugins_status', $result, 7*24 * HOUR_IN_SECONDS );
		}


	} else {  //Method not found
		//throw new Exception('{"error":{"code":-32601,"message":"Method not found","data":null},"id":"1"}');

		$dns = dns_get_record("user-api.simplybook.me", DNS_A);
		$pageDocument = @file_get_contents('https://user-api.simplybook.me');
		$pageDocument2 = @curl_load('https://user-api.simplybook.me');

		if ( !isset($dns[0]) || strlen($dns[0]['ip']) < 3) {
			throw new Exception('{"error":{"code":-32601,"message":"Seems the method cannot be found on your server. Possible reasons may include: PHP option \'allow_url_fopen\' is not enabled, PHP CURL is not enabled. Please ask your hoster for support regarding absence of required methods. We find error: DNS not worked","data":null},"id":"1"}');
		} else
		if( !ini_get('allow_url_fopen') ) {
			throw new Exception('{"error":{"code":-32601,"message":"Seems the method cannot be found on your server. Possible reasons may include: PHP option \'allow_url_fopen\' is not enabled, PHP CURL is not enabled. Please ask your hoster for support regarding absence of required methods. We find error: PHP option \'allow_url_fopen\' is disabled ","data":null},"id":"1"}');
		} else
		if( !_is_curl_installed() ) {
			throw new Exception('{"error":{"code":-32601,"message":"Seems the method cannot be found on your server. Possible reasons may include: PHP option \'allow_url_fopen\' is not enabled, PHP CURL is not enabled. Please ask your hoster for support regarding absence of required methods. We find error: PHP CURL is disabled ","data":null},"id":"1"}');
		} else
		if( $pageDocument === false ) {
			throw new Exception('{"error":{"code":-32601,"message":"Seems the method cannot be found on your server. Possible reasons may include: PHP option \'allow_url_fopen\' is not enabled, PHP CURL is not enabled. Please ask your hoster for support regarding absence of required methods. We find error: PHP function \'file_get_contents\' not worked ","data":null},"id":"1"}');
		} else
		if( strlen($pageDocument2) < 10 ) {
			throw new Exception('{"error":{"code":-32601,"message":"Seems the method cannot be found on your server. Possible reasons may include: PHP option \'allow_url_fopen\' is not enabled, PHP CURL is not enabled. Please ask your hoster for support regarding absence of required methods. We find error: PHP CURL not worked","data":null},"id":"1"}');
		} else {
			throw new Exception('{"error":{"code":-32601,"message":"Seems the method cannot be found on your server. Possible reasons may include: PHP option \'allow_url_fopen\' is not enabled, PHP CURL is not enabled. Please ask your hoster for support regarding absence of required methods. ","data":null},"id":"1"}');
		}
	}


} catch (Exception $e) {
	delete_transient( 'simplybook_token' );
	simplybook_clear_cache();

	echo $e->getMessage();
}


?>